#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun 19 15:19:29 2020

@author: davidworrall
version 0.01: Fri Jun 19 15:19:29 2020. Set up intitial cv read
version 0.02: 2020 31 Define class structure for a country
version 0.03: 2020 08 06 Add SMA
version 0.04: 2020 09 18 finish marked-line plotting routines * non-printing vals <- 0
version 0.05: 2020 09 19 finish x-axis 7day divisions and labels

Covid19/Coronavirus global deaths
dataset:
Title:    COVID-19 Time Series Data
Author:   Shad Reynolds
URL:      https://data.world/shad/covid-19-time-series-data

based on deaths_global.csv, as reguarly updated
online comments:
@deviramanan2016•3 months ago

@shad Do you have patient level covid19 data?
@jonathangill•last month

Jonathan Gill:  @jonathangill•last month
Jonathangill021@gmail.com:  @deeworr•last month

It would be very useful to have this - even if just basic data like sex and age.
-The US data is not geographically differentiated. I'm using https://github.com/nytimes/covid-19-data/blob/master/us-counties.csv . Does anyone have a better source?

- Hi Shad. Thanks for your work on this. One point and a question, if I may:
(a) The Country field for South Korea is in the form "Korea, South" which parses as two different values. Is it possible to change it to "South Korea"? makes the code less onerous.
(b) I notice there is a (weekly) "resonance" in most sequences. Do you know if this is a result of the collation methodologies? Dee.

The 2020 geographic population data is from the INTERNET USAGE STATISTICS website which also has land-area:
https://www.internetworldstats.com/stats.htm
https://www.internetworldstats.com/list2.htm#alpha


"""
# ====================IMPORT PROVINCE DATA AND STORE IN A DICT ===============#
# Some columnanmes to enhance readability
colLabelRow  = 0
stateCol     = 0
CountryCol   = 1
LatCol       = 2
LongCol      = 3
DateBeginCol = 4

import csv
directory="/Users/davidworrall/Documents/RESEARCH/COVID-19/"
directory="/volumes/DRWORANGE32/COVID-19/"
fname= "deaths_global.csv" # "us-counties20200516.csv"
#with open('/Users/davidworrall/Documents/CCC/TEACHING/TeachingResources_DSon/LiteracyRatesGlobalLongRun.csv', newline='') as csvfile:
# =========== PULL THE JOHNS Hopkins DB  FILE =================================
with open(directory+fname, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')
    deathsDB = []
    for row in ipdata:
        if row [CountryCol] == "Korea, South":                       # include 2 odd fixes
            row [CountryCol] = 'South Korea'
        if row [stateCol] == "Bonaire, Sint Eustatius and Saba":
            row [stateCol] = 'Bonaire Sint Eustatius and Saba'
        deathsDB += [[','.join(row)]]
for indx in range (1,len(deathsDB)):
    deathsDB[indx] = deathsDB[indx][0].split(',')                   # each row of the DB is indexed sequentially as per the original file
# convert death data to ints
    deathsDB[indx] [LatCol]    = float (deathsDB[indx] [LatCol])    # convert Latitude string to float
    deathsDB[indx] [LongCol]   = float (deathsDB[indx] [LongCol])   # convert Longditude string to float
    for deathNr in range (LongCol+1, len (deathsDB[indx])):         # convert the rest (death numbers) to ints
        deathsDB[indx][deathNr] = int (deathsDB[indx][deathNr])

deathsDB[colLabelRow] = deathsDB[colLabelRow][0].split(',')         # The column title texts/\
#           sunday      monday      tuesday   (wed = 1/22/20, the first date in the database, pre pad is used to startCharts on a Sun
datePad = ['1/19/20','1/20/20', '1/21/20']                          # 1/22/20 (the first date in the DB) is a Wed, so pre pad for a Sunday start
dataPad = [0, 0, 0]                                                 # insert len ( datePad days of 0 data)
columnLabels = deathsDB[0]                                          # a list of the column headings (dates etc)
#print ("============\nColumn Labels\n============\n", columnLabels)

dates = datePad + columnLabels[4:]                                  # Sunday-padded dates   #print (dates)
#print (dates)

deathsDB = deathsDB[1:]                                             # drop the column headings


def mmddyyyyy2yyyymmdd(spacer=""):
    """ converts US dates in DB (mm/dd/yy) to international (yyyySmmSdd)"""
    for indx in range (4,len(columnLabels)):
        #print (columnLabels [indx] , end= " ")                     # dates start at col 4
        tmp = columnLabels[indx].split('/')                         # [mm, dd, yy] as per the dataset
        columnLabels [indx]= ("20"+tmp[-1])+spacer+('0'+tmp[0])[-2:]+spacer+('0'+tmp[1])[-2:]  #[yyyySmmSdd]. S = spacer

mmddyyyyy2yyyymmdd(".")  # cseparate date components with a ","  # print (columnLabels)

deathsDB = deathsDB[1:]                                             # drop the column headings
#print ("============\nColumn Labels\n============\n", columnLabels)

countriesList = []              # collate all multiple-province country provinces into a single country entry
for province in deathsDB:       # make a lit of all countries with provinces
    if province[0] != '':
        if province[1] not in countriesList:
            countriesList += [province[1]]

for country in countriesList:
    runningDeathsSum = ['',country,'','']+ [0] * (len (columnLabels) - 4)    # we just want the death data clugey
    for province in deathsDB:       # find each province for that country
        if province [1] == country:
            for indx in range (4, len (runningDeathsSum)):
                runningDeathsSum [indx] = runningDeathsSum [indx] + province [indx]
#    print (runningDeathsSum)
    deathsDB += [runningDeathsSum]                                 #   add new country summation to the deathsDB

# if no province, make province == country
for prov in deathsDB:
    if prov[stateCol]=="":
        prov[stateCol] = prov [CountryCol]


provinceDeathsDict = {}                 # a dict to hold all country, province and summed-province data
for province in deathsDB:               # Collates all provinces into a single dictionary entry
    if province[0] == "":
        provinceDeathsDict [province[1]]  = province        # there is no province, the dict key is country
    else:
        provinceDeathsDict [province[0]]  = province        # there is more than on province so use the dick key is the province

"""
for item in provinceDeathsDict.keys():                      # prints and cross-checks OK!
#    print (provinceDeathsDict[item])
#print (len(provinceDeathsDict.keys()))
kees = sorted (provinceDeathsDict.keys())                   # alpha sort for pretty print
for keeNr in range (len (kees)):
    print (kees[keeNr], end="\t\t")
    if keeNr % 3 == 0:
        print()
"""
# ====================DEFINE DEATH CLASS FOR GLOBAL COUNTRIES/STATES?PROVINCES#

class CountryDeaths():    # define a class for handling Covid-19 death data for a country
    def __init__(self, countryName):
        """Initialize car attributes."""
        self.countryName        = countryName
        self.countryExists      = False
        self.printData          = False                 #used to filter death data when using print (instance)
        self.provinceName       = ""                    #used by __init__() to store the instance name
        self.latitude           = countriesList[LatCol]
        self.longitude          = countriesList[LatCol]
        self.area               = 0
        self.population         = 0
        self.populationDensity  = 0
        self.cumulativeDeaths   = []
        self.deathIncrements    = []   #sun     mon       tuea          wed
        self.prePadDates        = ['1/19/20','1/20/20', '1/21/20'] # 1/22/20 = first data date. prepad for a Sunday start
        self.prePadData         = [0,0,0]               # insert len ( self.prePadDates days of data = 0)
        self.NrDataPoints       = 0                     #Needed many time so no need to keep calculating it
        self.simpleMovingAveragePeriod = 0
        self.SMAdata            = []

        if self.countryName not in provinceDeathsDict.keys():
            self.countryExists = False
            print (self.countryName + " does not exist in the Database.\nIt contains data for the following countries:")
            kees = sorted (provinceDeathsDict.keys())    # alpha sort for 'pretty' print
            for keeNr in range (len (kees)):
                print (kees[keeNr], end="\t")
                if keeNr % 3 == 0: print()               # throw a CR
            return (None)
        else:
            self.countryExists = True
#            print("*********", provinceDeathsDict[self.countryName])
#            print
            self.provinceName   = provinceDeathsDict[countryName][0] #countriesList[self.countryName][0]
            self.latitude       = provinceDeathsDict[countryName][2] # countriesList[LatCol]
            self.longitude      = provinceDeathsDict[countryName][3] # countriesList[LongCol]
            self.area           = None
            self.population     = None
            self.cumulativeDeaths = provinceDeathsDict[countryName][4:]
            # block the printing of values <=0
            for indx in range (len(self.cumulativeDeaths)):
                if self.cumulativeDeaths[indx] <= 0:
                    self.cumulativeDeaths[indx] = float ('nan')     # nan = not-a-number
            #
            self.deathIncrements = [0]* len (self.cumulativeDeaths)
            for dayNr in range (1,len (self.cumulativeDeaths)):
                self.deathIncrements[dayNr] = self.cumulativeDeaths[dayNr]-self.cumulativeDeaths[dayNr-1]
                if self.deathIncrements[dayNr] <0:
                    print ("DATA ERROR:",self.countryName , end="\t")
#                    print ("DATA ERROR:Some dead rose again in ",self.countryName , "on", :", end="/t")
                    print (dayNr,":",columnLabels[dayNr+4], "\t", self.cumulativeDeaths[dayNr-1],"\t",self.cumulativeDeaths[dayNr],"\tdiff :", self.deathIncrements[dayNr-1],self.deathIncrements[dayNr])
                    self.deathIncrements[dayNr] = 0  # just zero it : #  ============ ERROR CORRECTION==============
            #
            if self.provinceName == "":
                self.provinceName = self.countryName                # the 'provine' is the country
            self.NrDataPoints = len (self.deathIncrements)

#            countriesInConsiderationList += [self.countryName]

    def __str__(self):
        """ Defines what to do when the default python srt() function is called on an instance"""
        if self.countryExists == True:
            print ('#' * 30)
            print ("Country:", self.countryName)
            print ('#' * 30)
            print ("Province:\t" , self.provinceName)
            print ("Latitude:\t" , self.latitude)
            print ("Longitude:\t" , self.longitude)
            print ("Area (sq kms):\t", self.area)
            print ("Population:\t" , self.population)
            if self.printData:
                print (self.countryName+": Cumulative Deaths:\n", self.cumulativeDeaths)
                print (self.countryName+": Death Increments:\n", self.deathIncrements)
        return ("")

    def SMA (self, NrPeriods):
        """ Computes a Simple Moving Average of self.deathIncrements over NrPeriods"""
        self.simpleMovingAveragePeriod = NrPeriods
        self.SMAdata = []
        lastDataPointIndex = len (self.deathIncrements)
        if lastDataPointIndex <  self.simpleMovingAveragePeriod:
            print ("Averaging period too long." +
                   "Longest period is: ", lastDataPointIndex-self.simpleMovingAveragePeriod)
            return(None)
        index = 0
        while index < self.NrDataPoints:
            thisWindow = self.deathIncrements[index: index+ self.simpleMovingAveragePeriod]
            windowAverage = sum(thisWindow) / self.simpleMovingAveragePeriod
#            print (thisWindow, windowAverage )
            self.SMAdata += [windowAverage]  # .append (windowAverage)
            index += 1
#        print (self.SMAdata)
 #       print (self.SMAdata)

# =================================
import matplotlib.pyplot as plt
# =================================
covidInstanceDict = {}   # used to  store country instances for plotting etc
def graphCountries(listOfCountryNames,graphName ):
    title = 'Daily Covid-19 Deaths for '+graphName+' for the period ' + columnLabels[4] + '—' + columnLabels[-1]
    plt.title (title, fontsize=14)
    plt.ylabel("Number of Confirmed Deaths", fontsize=10)
    plt.xlabel("Dates", fontsize=10)
    plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.15)
    plt.xticks (rotation = -60) #'vertical'
    xlabels = columnLabels[4:] # the dates. The first is Wednesday, Jan 22. Need to add 3 days to the beginning
#    print (xlabels)
    xmarkers = list (range(0,len(xlabels)+7, 7))
    xlabels2 = [xlabels[datey][-5:] for datey in range(0,len(xlabels), 7)]  # omot year- take just month and day
#    print (xmarkers)
    plt.xticks(xmarkers, xlabels2, fontsize=8)

#    plt.xticks(labels=xlabels )#, rotation = 90))
#    plt.xticks(ticks=xticks, labels=xlabels )#, rotation = 90))
    for country in listOfCountryNames:
#          instanceString = country +" = CountryDeaths('"+country+"')"
          instanceString = "covidInstanceDict['"+country+"'] = CountryDeaths('"+country+"')"
#          print ("Instance string:", instanceString)
          exec(instanceString)

          dgStr1 = "plt.plot(covidInstanceDict['"
          dgStr2 = "'].deathIncrements, label='"
          dgStr3 ="', marker='.', linewidth='0.5')"
          dataGraphString = dgStr1 +country+ dgStr2 +country+ dgStr3
#          print ('datagraphString:', dataGraphString)
          exec(dataGraphString)                         #  store a  created country instance in the country dictionaty

    plt.legend(title='Country/Region', loc = 'upper left', shadow = False)
    plt.show()
    datey = columnLabels[-1].replace(columnLabels[-1][4],"")    # take out the separator for the filename
    plt.savefig(directory+graphName+'_covid19_'+datey+'.jpg', transparent=True)

Iberia  = ['Portugal', 'Greece', 'Turkey']
Europe1 = ['Austria' , 'Portugal', 'Netherlands', 'Sweden', 'Finland']
Europe2 = ['United Kingdom', 'Spain', 'France', 'Italy', 'Greece']
EasternEurope = ['Poland', 'Hungary', 'Romania']              # EasternEurope,'Eastern Europe'
Americas = ['US' , 'Brazil', 'Mexico']                                  # Americas, 'The Americas'
Asia1 = ['India', 'Pakistan']


graphCountries(EasternEurope,'Eastern Europe')
"""
import numpy as np
#plt.title("Daily Covid-19 Deaths\nfor the period "+columnLabels[4]+ '–' +columnLabels[-1], fontsize=18)
#plt.ylabel("Number of Confirmed Deaths", fontsize=12)
#plt.legend(title='Country/Region')
#plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.25)
plt.xticks (np.arange (0,brazil.NrDataPoints+1, 7),rotation=90)
plt.yticks(np.arange(0,2700,200) )
plt.show()




# test

for country in countriesInConsiderationList:

    print (eval(country))
#print (countriesInConsiderationList[0]+'.'+'provinceName')
for prov in deathsDB[10:40]:
    instanceName = prov[0]
    exec ("instanceName = CountryDeaths(instanceName)")
    countries2graph += [instanceName]
##    exec ("instanceName = CountryDeaths(instanceName)")
#    plt.plot (eval ("instanceName.deathIncrements"))
#plt.show()
#plt.close()

print ("FIN")
#plt.close()

# print (countriesInConsiderationList)

#UK.printData=False ;
#aust.printData=True ;
#for prov in countriesInConsiderationList:
#    plt.plot (eval (prov+".deathIncrements"))


"""

#print (spain.deathIncrements)

"""
from datetime import datetime

for datey in range (DateBeginCol, len(deathsDB[colLabelRow])):
    deathsDB[colLabelRow][datey] = datetime.strptime(deathsDB[colLabelRow][datey], "%m/%d/%y").date()
#    print (deathsDB[colLabelRow][datey])

for datey in range (DateBeginCol, len(deathsDB[colLabelRow])-7, 4):
    print (deathsDB[colLabelRow][datey+7]- deathsDB[colLabelRow][datey])
    print (deathsDB[colLabelRow][datey])


dateDict={}


countyList = []

dateList = []
for place in deathsDB:
    if place[0] not in dateList:
        dateList += [place[0]]
dateList = dateList[1:]                                     # drop the column title
#print (dateList)
#print ("==========dates\n", dateList)
"""
"""plt.title("Daily Covid-19 Deaths\nfor the period "+columnLabels[4]+ '–' +columnLabels[-1], fontsize=18)
plt.ylabel("Number of Confirmed Deaths", fontsize=12)
plt.legend(title='Country/Region')
plt.grid(axis='both', which='both', color='grey', linestyle='-', linewidth=0.25)
"""
    # tests of class CountryDeaths()
"""

aust = CountryDeaths("Australia")       #; countriesInConsiderationList += ['aust']
nz=CountryDeaths("New Zealand")
iceland = CountryDeaths("Iceland")
china = CountryDeaths("China")
plt.plot(china.deathIncrements)
plt.plot(china.SMAdata)
china.SMA(3)
plt.plot(china.SMAdata)
brazil =    CountryDeaths('Brazil') ; plt.plot(brazil.deathIncrements,    label='Brazil') # ; brazil.SMA(3) ; plt.plot(brazil.SMAdata)
china =     CountryDeaths('China') ; plt.plot(brazil.deathIncrements,    label='China') # ; brazil.SMA(3) ; plt.plot(brazil.SMAdata)
france =    CountryDeaths('France')  ; plt.plot(france.deathIncrements,     label='France')
india =     CountryDeaths('India')  ; plt.plot(india.deathIncrements,     label='India')
mexico =    CountryDeaths('Mexico') ; plt.plot(mexico.deathIncrements,    label='Mexico')
#pakistan =  CountryDeaths('Pakistan'); plt.plot(pakistan.deathIncrements,  label='Pakistan')
india =     CountryDeaths('India')  ; plt.plot(india.deathIncrements,     label='India')
russia =    CountryDeaths('Russia") ; plt.plot(russia.deathIncrements,    label='Russia')
spain =     CountryDeaths('Spain") ; plt.plot(spain.deathIncrements,    label='Spain")
#sweden =    CountryDeaths("Sweden") ; plt.plot(sweden.deathIncrements,    label='Sweden')
UK =        CountryDeaths('United Kingdom") ; plt.plot(UK.deathIncrements,  label='United Kingdom')
USA =       CountryDeaths('USA')     ; plt.plot(USA.deathIncrements,       label='United States')
"""
